/*
 * Solidarity Mobility API
 *
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * API version: 1.0.0
 * Generated by: OpenAPI Generator (https://openapi-generator.tech)
 */

package openapi

import (
	"context"
	"fmt"
	"github.com/spf13/viper"
	"net/http"
	"solidarity-service/handler"
	"solidarity-service/internal"
	"solidarity-service/storage"
	"strings"
)

// BookingsFilterAPIService is a service that implements the logic for the BookingsFilterAPIServicer
// This service should implement the business logic for every endpoint for the BookingsFilterAPI API.
// Include any external packages or services that will be required by this service.
type BookingsFilterAPIService struct {
	config  *viper.Viper
	handler *handler.SolidarityServiceHandler
	storage storage.Storage
}

// NewBookingsFilterAPIService creates a default api service
func NewBookingsFilterAPIService(config *viper.Viper, handler *handler.SolidarityServiceHandler, storage storage.Storage) BookingsFilterAPIServicer {
	return &BookingsFilterAPIService{
		config:  config,
		handler: handler,
		storage: storage,
	}
}

// FilterBookingsByStatus - Filter and retrieve bookings by status, type, and ID.
func (s *BookingsFilterAPIService) FilterBookingsByStatus(ctx context.Context, operator string, status string, filterType string, id string) (ImplResponse, error) {
	bookings, err := s.storage.FilterUserBookingsByStatus(filterType, internal.BookingStatus(status), id)
	if err != nil {
		fmt.Println(err)
		if strings.Contains(err.Error(), " no rows in result set") {
			return Response(http.StatusBadRequest, "ID not found in the database"), nil
		} else {
			return Response(http.StatusInternalServerError, nil), nil
		}
	}
	responses := []Booking{}
	for _, v := range bookings {
		passenger, err := s.storage.GetPassenger(v.Passenger.ID)
		if err != nil {
			return Response(http.StatusInternalServerError, nil), nil
		}
		driver, err := s.storage.GetDriver(v.Driver.ID)
		if err != nil {
			return Response(http.StatusInternalServerError, nil), nil
		}
		duration, err := s.handler.CalculateDurationBetweenFeatures(passenger.Passenger_departure_address, passenger.Passenger_destination_address)
		if err != nil {
			duration = 0
		}
		responses = append(responses, Booking{
			Id:     v.ID,
			Status: BookingStatus(v.Status),
			Driver: User{
				Id:               v.Driver.ID,
				Operator:         driver.Driver.Operator,
				Alias:            driver.Driver.Alias,
				FirstName:        driver.Driver.FirstName,
				LastName:         driver.Driver.LastName,
				Grade:            int32(driver.Driver.Grade),
				Picture:          driver.Driver.Picture,
				Gender:           driver.Driver.Gender,
				VerifiedIdentity: driver.Driver.VerifiedIdentity,
			},
			Passenger: User{
				Id:               v.Passenger.ID,
				Operator:         passenger.Passenger.Operator,
				Alias:            passenger.Passenger.Alias,
				FirstName:        passenger.Passenger.FirstName,
				LastName:         passenger.Passenger.LastName,
				Grade:            int32(passenger.Passenger.Grade),
				Picture:          passenger.Passenger.Picture,
				Gender:           passenger.Passenger.Gender,
				VerifiedIdentity: passenger.Passenger.VerifiedIdentity,
			},
			PassengerPickupDate:    int32(passenger.Passenger_pickup_date),
			PassengerPickupLat:     passenger.Passenger_departure_address.Point().X(),
			PassengerPickupLng:     passenger.Passenger_departure_address.Point().Y(),
			PassengerPickupAddress: passenger.Passenger_departure_address.Properties.MustString("name"),
			PassengerDropLat:       passenger.Passenger_destination_address.Point().X(),
			PassengerDropLng:       passenger.Passenger_destination_address.Point().Y(),
			PassengerDropAddress:   passenger.Passenger_destination_address.Properties.MustString("name"),
			Duration:               int32(duration),
			Distance:               int32(s.handler.CalculateDistanceBetweenFeatures(passenger.Passenger_departure_address, passenger.Passenger_destination_address)),
			Car: Car{
				Model: driver.Car.Model,
				Brand: driver.Car.Brand,
			},
			Price: Price{
				Type: "FREE",
			},
		})
	}

	return Response(200, responses), nil
}
